const writeFile = require('node:fs').writeFile;
const opendir = require('node:fs').opendir;
const mkdir = require('node:fs').mkdir;
const jsonFormat = require('json-format');

function setVue3DevTemplate() {
    const jsonIndentConfig = {
        type: 'space',
        size: 4,
    };
    const snippetsDir = './.vscode';

    const scopeVue = 'vue';
    const scopeJS = 'javascript';
    const snippets = {
        '<vue3>, Setup 语法糖, Vue3': {
            scope: scopeVue,
            prefix: '<vue3>',
            body: [
                `<template>`,
                `    `,
                `</template>`,
                ``,
                `<script setup>`,
                `onMounted(() => {`,
                `    `,
                `});`,
                ``,
                `onUnmounted(() => {`,
                `    `,
                `});`,
                `</script>`,
                ``,
                `<style lang="less" scoped>`,
                `    `,
                `</style>`,
                `<style lang="less">`,
                `    `,
                `</style>`,
            ],
            description: '<vue3>, Setup 语法糖, Vue3',
        },
        '<script setup>, Setup 语法糖, Vue3': {
            scope: scopeVue,
            prefix: '<script setup>',
            body: [
                `<script setup>`,
                `onMounted(() => {`,
                `    `,
                `});`,
                ``,
                `onUnmounted(() => {`,
                `    `,
                `});`,
                `</script>`,
            ],
            description: '<script setup>, Setup 语法糖, Vue3',
        },
        '声明 props, 表达式, Vue3': {
            scope: scopeJS,
            prefix: 'def props',
            body: [
                `const props = defineProps({`,
                `    $1`,
                `});`,
            ],
            description: '声明 props, 表达式, Vue3',
        },
        '声明 provide, 表达式, Vue3': {
            scope: scopeJS,
            prefix: 'def provide',
            body: [
                `provide(\$\{1:alias\}, \$\{2:data\});`,
            ],
            description: '声明 provide, 表达式, Vue3',
        },
        '声明 inject, 表达式, Vue3': {
            scope: scopeJS,
            prefix: 'def inject',
            body: [
                `const \$\{1:injectData\} = inject(\$\{2:alias\});`,
            ],
            description: '声明 inject, 表达式, Vue3',
        },
        '声明 emits, 表达式, Vue3': {
            scope: scopeJS,
            prefix: 'def emits',
            body: [
                `const emit = defineEmits([ $1 ]);`,
            ],
            description: '声明 emits, 表达式, Vue3',
        },
        '声明 ref, 表达式, Vue3': {
            scope: scopeJS,
            prefix: 'def ref',
            body: [
                `const \$\{1:refName\} = ref(\$\{2:initValue\});`,
            ],
            description: '声明 ref, 表达式, Vue3',
        },
        '声明 computed, 表达式, Vue3': {
            scope: scopeJS,
            prefix: 'def computed',
            body: [
                `const \$\{1:computedName\} = computed(() => \$\{2:expression\});`,
            ],
            description: '声明 computed, 表达式, Vue3',
        },
        '声明 watch, 表达式, Vue3': {
            scope: scopeJS,
            prefix: 'def watch',
            body: [
                `watch(`,
                `    () => \$\{1:expression\},`,
                `    (newValue, oldValue) => {`,
                `        $2`,
                `    },`,
                `);`,
            ],
            description: '声明 watch, 表达式, Vue3',
        },
        '声明 function, 表达式, Vue3': {
            scope: scopeJS,
            prefix: 'def function',
            body: [
                `function \$\{1:functionName\}() {`,
                `    $2`,
                `}`,
            ],
            description: '声明 function, 表达式, Vue3',
        },
        '声明 mounted, 表达式, Vue3': {
            scope: scopeJS,
            prefix: 'def mounted',
            body: [
                `onMounted(() => {`,
                `    $1`,
                `});`,
            ],
            description: '声明 mounted, 表达式, Vue3',
        },
        '声明 unmounted, 表达式, Vue3': {
            scope: scopeJS,
            prefix: 'def unmounted',
            body: [
                `onUnmounted(() => {`,
                `    $1`,
                `});`,
            ],
            description: '声明 unmounted, 表达式, Vue3',
        },
        '声明 expose, 表达式, Vue3': {
            scope: scopeJS,
            prefix: 'def expose',
            body: [
                `defineExpose({`,
                `    $1`,
                `});`,
            ],
            description: '声明 expose, 表达式, Vue3',
        },
        '声明 Vuex store, 表达式, Vuex': {
            scope: scopeJS,
            prefix: 'def store',
            body: [
                `const store = useStore();`,
            ],
            description: '声明 Vuex store, 表达式, Vuex',
        },
        '声明 Vue Route, 表达式, Vue-Router': {
            scope: scopeJS,
            prefix: 'def route',
            body: [
                `const route = useRoute();`,
            ],
            description: '声明 Vue Route, 表达式, Vue-Router',
        },
        '声明 Vue Router, 表达式, Vue-Router': {
            scope: scopeJS,
            prefix: 'def router',
            body: [
                `const router = useRouter();`,
            ],
            description: '声明 Vue Router, 表达式, Vue-Router',
        },
    }
    const json = jsonFormat(
        snippets,
        jsonIndentConfig
    );

    function open() {
        return new Promise((resolve, reject) => {
            opendir(snippetsDir, (error) => {
                if (error) {
                    reject(error);
                } else {
                    resolve();
                }
            });
        });
    }
    function make() {
        return new Promise((resolve, reject) => {
            mkdir(snippetsDir, { recursive: true }, (error) => {
                if (error) {
                    reject(error);
                } else {
                    resolve();
                }
            });
        });
    }
    function write() {
        writeFile(
            `${snippetsDir}/vue3-dev-template.code-snippets`,
            json,
            'utf8',
            (error) => {
                if (error) throw error;
                console.log('done');
            }
        );
    }
    open()
        .then(() => {
            write();
        })
        .catch((error) => {
            make()
                .then(() => {
                    write();
                })
                .catch((error) => {
                    console.error(error);
                });
        });

    return void 0;
}
setVue3DevTemplate();
